<?php
class CheckissuingAPIClient {
  public $opts;
  protected $ch;
  protected $base_url;
  protected $prod_base_url = 'https://api2.checkissuing.com/';
  protected $sb_base_url = 'https://api2sb.checkissuing.com/';
  protected $curl_opts = array();

  protected $testing = true;
  protected $response_format = 'json';

  protected $client_id = null;
  protected $api_key = null;

  public $lastErrors = null;
  public $lastResponse = null;

  public $accessToken = null;

  function __construct($opts = array()) {
      $this->opts = $opts;

      $this->base_url = '';
      $this->curl_opts = array(
        CURLOPT_AUTOREFERER => true,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_POST => true
      );

      foreach ($this->opts as $k => $v) {
        switch ($k) {
          case 'curl_opts':
            if (is_array($v)) {
              foreach ($v as $cok => $cov) {
                $this->curl_opts[$cok] = $cov;
              }
            }
            break;

          default:
            if (property_exists($this, $k)) {
              $this->$k = $v;
            }
            break;
        }
      }

      if ($this->testing) {
        $this->base_url = $this->sb_base_url;
      } else {
        $this->base_url = $this->prod_base_url;
      }

      $this->ch = curl_init($this->base_url);
      curl_setopt_array($this->ch, $this->curl_opts);
  }

  public function exec($endpoint, $opts = array(), $method = 'POST', $raw = false) {
    if (!stristr($endpoint, '://')) {
      $url = $this->base_url . $endpoint;
    }

    $url_str = '';
    curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, $method);

	if (stristr($endpoint, 'access-token') || stristr($endpoint, 'access_token')) {
		if (isset($opts['api_id']) && isset($opts['api_secret'])) {
			$api_id = $opts['api_id'];
			$api_secret = $opts['api_secret'];

			unset($opts['api_id']);
			unset($opts['api_secret']);

			curl_setopt($this->ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
			curl_setopt($this->ch, CURLOPT_USERPWD, $api_id . ':' . $api_secret);

			if (!isset($opts['grant_type'])) {
				$opts['grant_type'] = 'client_credentials';
			}
		}
	} else {
		curl_setopt($this->ch, CURLINFO_HEADER_OUT,1);
    curl_setopt($this->ch, CURLOPT_HTTPHEADER, array(
   			'Authorization: Bearer ' . $this->accessToken['access_token']
   		));


	}

    switch ($method) {
      case 'POST':
        curl_setopt($this->ch, CURLOPT_POSTFIELDS, http_build_query($opts));
        break;

      case 'GET':
        if (!empty($opts)) {
          $url_str = '?';

          $ii = 0;
          foreach ($opts as $k => $v) {
            if ($ii > 0) {
              $url_str .= '&';
            }

            $url_str .= urlencode($k) . '=' . urlencode($v);
            $ii++;
          }
        }
        break;
    }

    curl_setopt($this->ch, CURLOPT_URL, $url . $url_str);

	$result = curl_exec($this->ch);
  $cinfo = curl_getinfo($this->ch);

    if ($raw) {
      return $result;
    }
    
    if (!strcmp($this->response_format, 'json')) {
      $tmp = @json_decode($result, true);

      $errors = array();

      switch (json_last_error()) {
          case JSON_ERROR_NONE:
              //return ' - No errors';
          break;
          case JSON_ERROR_DEPTH:
              $errors[] = 'Maximum stack depth exceeded';
          break;
          case JSON_ERROR_STATE_MISMATCH:
              $errors[] = 'Underflow or the modes mismatch';
          break;
          case JSON_ERROR_CTRL_CHAR:
              $errors[] = 'Unexpected control character found';
          break;
          case JSON_ERROR_SYNTAX:
              $errors[] = 'Syntax error, malformed JSON';
          break;
          case JSON_ERROR_UTF8:
              $errors[] = 'Malformed UTF-8 characters, possibly incorrectly encoded';
          break;
          default:
              $errors[] = 'Unknown error';
          break;
      }

      if (is_array($tmp)) {
        $tmp['raw_response'] = $result;
        $this->lastResponse = $tmp;
        if (!empty($tmp['errors'])) {
          $this->lastErrors = $tmp['errors'];
        }
        return $tmp;
      } else {
        return array('status' => 0, 'raw_response' => $result, 'errors' => array_merge(array('Error parsing response'), $errors), 'errorMessage' => 'Error parsing response' . implode("|", $errors));
      }
    } else {
      $tmp = simplexml_load_string($result);

      if (is_object($tmp)) {
        return $tmp;
      } else {
        return array('status' => 0, 'raw_response' => $result, 'errors' => array('Error parsing response'), 'errorMessage' => 'Error parsing response');
      }
    }
  }

  public function getAccessToken($api_id, $api_secret) {
	  $res = $this->exec('oauth/access-token', array('api_id' => $api_id, 'api_secret' => $api_secret, 'grant_type' => 'client_credentials'));

	  if (isset($res['access_token'])) {
		  $this->accessToken = $res;
	  }
	  return $res;
  }
}

class CheckissuingAPICollection {
  protected $api;
  protected $endpoint = null;

  protected $class = null;

  public function __construct($api, $class = null, $endpoint = null) {
    $this->api = $api;
    $this->class = null;
    $this->endpoint = $endpoint;
  }

  public function delete($id) {
    $res = $this->api->exec($this->endpoint . '/delete', array('id' => $id));

    return ($res['status'] == 1);
  }
}

class CheckissuingAPIObject {
  protected $api;
  protected $endpoint = null;
  protected $collection = null;

  public function __construct($api, $collection = null, $endpoint = null) {
    $this->api = $api;
    $this->collection = $collection;
    $this->endpoint = $endpoint;
  }
}

class CheckissuingBankAccounts extends CheckissuingAPICollection {
  public function __construct($api) {
    parent::__construct($api, 'CheckissuingBankAccount', 'bank-accounts');
  }

  public function add($opts) {
    $res = $this->api->exec($this->endpoint . '/add', $opts);

    if ($res['status'] == 1) {
      return $res['onboard_uri'];
    } else {
      return FALSE;
    }
  }

  public function get($id = null) {
    if (!is_null($id) && is_numeric($id)) {
      $res = $this->api->exec($this->endpoint . '/by-id', array('id' => $id));

      if ($res['status'] == 1) {
        $result = new CheckissuingBankAccount($this->api, $this);
        $result->setData($res['account']);

        return $result;
      } else {
        return FALSE;
      }
    } else {
      $res = $this->api->exec($this->endpoint);

      if ($res['status'] == 1) {
        $result = array();

        foreach ($res['accounts'] as $aid => $a) {
          $result[$aid] = new CheckissuingBankAccount($this->api, $this);
          $result[$aid]->setData($a);
        }

        return $result;
      } else {
        return FALSE;
      }
    }
  }
}

class CheckissuingBankAccount extends CheckissuingAPIObject {
  protected $data = array();

  public function __construct($api, $collection = null) {
    parent::__construct($api, $collection, 'bank-accounts');
  }

  public function setData($data) {
    $this->data = $data;
  }

  public function metadata($data = null, $delete_on_empty = false) {
    $params = array('delete_empty' => ($delete_on_empty ? 1 : 0));

    $setting = false;

    if (!empty($data) && is_array($data)) {
      $setting = true;
      $params['data'] = $data;
    }

    $res = $this->api->exec($this->endpoint . '/metadata', $params);

    if ($setting) {
      return ($res['status'] == 1);
    } else {
      if ($res['status'] == 1) {
        return $res['account']['metadata'];
      } else {
        return FALSE;
      }
    }
  }

  public function __get($k) {
    if (property_exists($this, $k)) {
      return $this->$k;
    } elseif (isset($this->data[$k])) {
      return $this->data[$k];
    } else {
      return FALSE;
    }
  }

  public function __set($k, $v) {
    if (property_exists($this, $k)) {
      $this->$k = $v;
    } else {
      $this->data[$k] = $v;
    }

    return TRUE;
  }

  public function verify($id = null) {
    $res = $this->api->exec($this->endpoint, array('id' => (isset($this->data['id']) ? $this->data['id'] : $id)));

    if ($res['status'] == 1) {
      return $res['verify_uri'];
    } else {
      return FALSE;
    }
  }
}

class CheckissuingPayments extends CheckissuingAPICollection {
  public function __construct($api) {
    parent::__construct($api, 'CheckissuingPayment', 'payments');
  }

  public function add($payment) {
    $res = $this->api->exec($this->endpoint . '/add', array('payment' => $payment));

    if ($res['status'] == 1) {
      $p = new CheckissuingPayment($this->api, $this);
      $p->setData($res['payment']);

      return $p;
    } else {
      return $res['errors'];
    }
  }

  public function get($id = null) {
    if (!is_null($id) && is_numeric($id)) {
      $res = $this->api->exec($this->endpoint . '/by-id', array('id' => $id));

      if ($res['status'] == 1) {
        $result = new CheckissuingPayment($this->api, $this);
        $result->setData($res['payment']);

        return $result;
      } else {
        return FALSE;
      }
    } else {
      $res = $this->api->exec($this->endpoint);

      if ($res['status'] == 1) {
        $result = array();

        foreach ($res['payments'] as $pid => $p) {
          $result[$pid] = new CheckissuingPayment($this->api, $this);
          $result[$pid]->setData($p);
        }

        return $result;
      } else {
        return FALSE;
      }
    }
  }

  public function delete($id) {
    $res = $this->api->exec($this->endpoint . '/delete', array('id' => $id));

    if ($res['status'] == 1) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  public function upload_file($file_path) {
    $res = $this->api->exec($this->endpoint . '/upload', array('userfile' => new CURLFile($file_path)));

    if ($res['status'] == 1) {
      return $res['file_id'];
    } else {
      return FALSE;
    }
  }

  public function confirm($id, $bundle_opts = array()) {
    $res = $this->api->exec($this->endpoint . '/confirm', array_merge(array('id' => array($id)), $bundle_opts));

    return ($res['status'] == 1);
  }
}

class CheckissuingPayment extends CheckissuingAPIObject {
  protected $data = array();

  public function __construct($api, $collection = null) {
    parent::__construct($api, $collection, 'payments');
  }

  public function setData($data) {
    $this->data = $data;
  }

  public function __get($k) {
    if (property_exists($this, $k)) {
      return $this->$k;
    } elseif (isset($this->data[$k])) {
      return $this->data[$k];
    } else {
      return FALSE;
    }
  }

  public function __set($k, $v) {
    if (property_exists($this, $k)) {
      $this->$k = $v;
    } else {
      $this->data[$k] = $v;
    }

    return TRUE;
  }

  public function cancel($reason = '', $id = null) {
    $res = $this->api->exec($this->endpoint, array('id' => (isset($this->data['id']) ? $this->data['id'] : $id), 'reason' => $reason));

    if ($res['status'] == 1) {
      return TRUE;
    } else {
      return FALSE;
    }
  }
}
?>
